@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,8 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /* maintaince of mysql databases */
 
@@ -24,14 +23,9 @@
 #include <sys/stat.h>
 #include <mysql.h>
 #include <sql_common.h>
-#include <welcome_copyright_notice.h>           /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-#include "../ndb/src/mgmclient/ndb_mgmclient.h"
-#endif
-
-#define ADMIN_VERSION "8.41"
-#define MAX_MYSQL_VAR 256
+#define ADMIN_VERSION "8.42"
+#define MAX_MYSQL_VAR 512
 #define SHUTDOWN_DEF_TIMEOUT 3600		/* Wait for shutdown */
 #define MAX_TRUNC_LENGTH 3
 
@@ -43,20 +37,18 @@ ulonglong last_values[MAX_MYSQL_VAR];
 static int interval=0;
 static my_bool option_force=0,interrupted=0,new_line=0,
                opt_compress=0, opt_relative=0, opt_verbose=0, opt_vertical=0,
-               tty_password=0;
-static uint tcp_port = 0, option_wait = 0, option_silent=0, nr_iterations,
-            opt_count_iterations= 0;
+               tty_password= 0, opt_nobeep;
+static my_bool debug_info_flag= 0, debug_check_flag= 0;
+static uint tcp_port = 0, option_wait = 0, option_silent=0, nr_iterations;
+static uint opt_count_iterations= 0, my_end_arg;
 static ulong opt_connect_timeout, opt_shutdown_timeout;
-static my_string unix_port=0;
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-static my_bool opt_ndbcluster=0;
-static char *opt_ndb_connectstring=0;
-#endif
+static char * unix_port=0;
 
 #ifdef HAVE_SMEM
 static char *shared_memory_base_name=0;
 #endif
 static uint opt_protocol=0;
+static myf error_flags; /* flags to pass to my_printf_error, like ME_BELL */
 
 /*
   When using extended-status relatively, ex_val_max_len is the estimated
@@ -72,10 +64,12 @@ static uint ex_var_count, max_var_length, max_val_length;
 
 static void print_version(void);
 static void usage(void);
+extern "C" my_bool get_one_option(int optid, const struct my_option *opt,
+                                  char *argument);
 static my_bool sql_connect(MYSQL *mysql, uint wait);
 static int execute_commands(MYSQL *mysql,int argc, char **argv);
 static int drop_db(MYSQL *mysql,const char *db);
-static sig_handler endprog(int signal_number);
+extern "C" sig_handler endprog(int signal_number);
 static void nice_time(ulong sec,char *buff);
 static void print_header(MYSQL_RES *result);
 static void print_top(MYSQL_RES *result);
@@ -104,9 +98,6 @@ enum commands {
   ADMIN_PING,             ADMIN_EXTENDED_STATUS, ADMIN_FLUSH_STATUS,
   ADMIN_FLUSH_PRIVILEGES, ADMIN_START_SLAVE,     ADMIN_STOP_SLAVE,
   ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  ,ADMIN_NDB_MGM
-#endif
 };
 static const char *command_names[]= {
   "create",               "drop",                "shutdown",
@@ -117,9 +108,6 @@ static const char *command_names[]= {
   "ping",                 "extended-status",     "flush-status",
   "flush-privileges",     "start-slave",         "stop-slave",
   "flush-threads","old-password",
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  "ndb-mgm",
-#endif
   NullS
 };
 
@@ -129,32 +117,42 @@ static TYPELIB command_typelib=
 static struct my_option my_long_options[] =
 {
 #ifdef __NETWARE__
-  {"autoclose", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
+  {"autoclose", OPT_AUTO_CLOSE, "Automatically close the screen on exit for Netware.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
   {"count", 'c',
    "Number of iterations to make. This works with -i (--sleep) only.",
-   (gptr*) &nr_iterations, (gptr*) &nr_iterations, 0, GET_UINT,
+   (uchar**) &nr_iterations, (uchar**) &nr_iterations, 0, GET_UINT,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+#ifndef DBUG_OFF
   {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
+#endif
+  {"debug-check", OPT_DEBUG_CHECK, "Check memory and open file usage at exit.",
+   (uchar**) &debug_check_flag, (uchar**) &debug_check_flag, 0,
+   GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+  {"debug-info", OPT_DEBUG_INFO, "Print some debug info at exit.",
+   (uchar**) &debug_info_flag, (uchar**) &debug_info_flag,
+   0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"force", 'f',
    "Don't ask for confirmation on drop database; with multiple commands, continue even if an error occurs.",
-   (gptr*) &option_force, (gptr*) &option_force, 0, GET_BOOL, NO_ARG, 0, 0,
+   (uchar**) &option_force, (uchar**) &option_force, 0, GET_BOOL, NO_ARG, 0, 0,
    0, 0, 0, 0},
   {"compress", 'C', "Use compression in server/client protocol.",
-   (gptr*) &opt_compress, (gptr*) &opt_compress, 0, GET_BOOL, NO_ARG, 0, 0, 0,
+   (uchar**) &opt_compress, (uchar**) &opt_compress, 0, GET_BOOL, NO_ARG, 0, 0, 0,
    0, 0, 0},
   {"character-sets-dir", OPT_CHARSETS_DIR,
-   "Directory where character sets are.", (gptr*) &charsets_dir,
-   (gptr*) &charsets_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+   "Directory for character set files.", (uchar**) &charsets_dir,
+   (uchar**) &charsets_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"default-character-set", OPT_DEFAULT_CHARSET,
-   "Set the default character set.", (gptr*) &default_charset,
-   (gptr*) &default_charset, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+   "Set the default character set.", (uchar**) &default_charset,
+   (uchar**) &default_charset, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"host", 'h', "Connect to host.", (gptr*) &host, (gptr*) &host, 0, GET_STR,
+  {"host", 'h', "Connect to host.", (uchar**) &host, (uchar**) &host, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"no-beep", 'b', "Turn off beep on error.", (uchar**) &opt_nobeep,
+   (uchar**) &opt_nobeep, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0}, 
   {"password", 'p',
    "Password to use when connecting to server. If password is not given it's asked from the tty.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
@@ -168,66 +166,58 @@ static struct my_option my_long_options[] =
    "/etc/services, "
 #endif
    "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
-   (gptr*) &tcp_port,
-   (gptr*) &tcp_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-  {"protocol", OPT_MYSQL_PROTOCOL, "The protocol of connection (tcp,socket,pipe,memory).",
+   (uchar**) &tcp_port,
+   (uchar**) &tcp_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"protocol", OPT_MYSQL_PROTOCOL, "The protocol to use for connection (tcp, socket, pipe, memory).",
     0, 0, 0, GET_STR,  REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"relative", 'r',
-   "Show difference between current and previous values when used with -i. Currently works only with extended-status.",
-   (gptr*) &opt_relative, (gptr*) &opt_relative, 0, GET_BOOL, NO_ARG, 0, 0, 0,
+   "Show difference between current and previous values when used with -i. Currently only works with extended-status.",
+   (uchar**) &opt_relative, (uchar**) &opt_relative, 0, GET_BOOL, NO_ARG, 0, 0, 0,
   0, 0, 0},
   {"set-variable", 'O',
    "Change the value of a variable. Please note that this option is deprecated; you can set variables directly with --variable-name=value.",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #ifdef HAVE_SMEM
   {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
-   "Base name of shared memory.", (gptr*) &shared_memory_base_name, (gptr*) &shared_memory_base_name,
+   "Base name of shared memory.", (uchar**) &shared_memory_base_name, (uchar**) &shared_memory_base_name,
    0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #endif
   {"silent", 's', "Silently exit if one can't connect to server.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"socket", 'S', "Socket file to use for connection.",
-   (gptr*) &unix_port, (gptr*) &unix_port, 0, GET_STR, REQUIRED_ARG, 0, 0, 0,
+  {"socket", 'S', "The socket file to use for connection.",
+   (uchar**) &unix_port, (uchar**) &unix_port, 0, GET_STR, REQUIRED_ARG, 0, 0, 0,
    0, 0, 0},
-  {"sleep", 'i', "Execute commands again and again with a sleep between.",
-   (gptr*) &interval, (gptr*) &interval, 0, GET_INT, REQUIRED_ARG, 0, 0, 0, 0,
+  {"sleep", 'i', "Execute commands repeatedly with a sleep between.",
+   (uchar**) &interval, (uchar**) &interval, 0, GET_INT, REQUIRED_ARG, 0, 0, 0, 0,
    0, 0},
 #include <sslopt-longopts.h>
 #ifndef DONT_ALLOW_USER_CHANGE
-  {"user", 'u', "User for login if not current user.", (gptr*) &user,
-   (gptr*) &user, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"user", 'u', "User for login if not current user.", (uchar**) &user,
+   (uchar**) &user, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #endif
-  {"verbose", 'v', "Write more information.", (gptr*) &opt_verbose,
-   (gptr*) &opt_verbose, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+  {"verbose", 'v', "Write more information.", (uchar**) &opt_verbose,
+   (uchar**) &opt_verbose, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"vertical", 'E',
    "Print output vertically. Is similar to --relative, but prints output vertically.",
-   (gptr*) &opt_vertical, (gptr*) &opt_vertical, 0, GET_BOOL, NO_ARG, 0, 0, 0,
+   (uchar**) &opt_vertical, (uchar**) &opt_vertical, 0, GET_BOOL, NO_ARG, 0, 0, 0,
    0, 0, 0},
   {"wait", 'w', "Wait and retry if connection is down.", 0, 0, 0, GET_UINT,
    OPT_ARG, 0, 0, 0, 0, 0, 0},
-  {"connect_timeout", OPT_CONNECT_TIMEOUT, "", (gptr*) &opt_connect_timeout,
-   (gptr*) &opt_connect_timeout, 0, GET_ULONG, REQUIRED_ARG, 3600*12, 0,
+  {"connect_timeout", OPT_CONNECT_TIMEOUT, "", (uchar**) &opt_connect_timeout,
+   (uchar**) &opt_connect_timeout, 0, GET_ULONG, REQUIRED_ARG, 3600*12, 0,
    3600*12, 0, 1, 0},
-  {"shutdown_timeout", OPT_SHUTDOWN_TIMEOUT, "", (gptr*) &opt_shutdown_timeout,
-   (gptr*) &opt_shutdown_timeout, 0, GET_ULONG, REQUIRED_ARG,
+  {"shutdown_timeout", OPT_SHUTDOWN_TIMEOUT, "", (uchar**) &opt_shutdown_timeout,
+   (uchar**) &opt_shutdown_timeout, 0, GET_ULONG, REQUIRED_ARG,
    SHUTDOWN_DEF_TIMEOUT, 0, 3600*12, 0, 1, 0},
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  {"ndbcluster", OPT_NDBCLUSTER, ""
-   "", (gptr*) &opt_ndbcluster,
-   (gptr*) &opt_ndbcluster, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"ndb-connectstring", OPT_NDB_CONNECTSTRING, ""
-   "", (gptr*) &opt_ndb_connectstring,
-   (gptr*) &opt_ndb_connectstring, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#endif
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
 
 static const char *load_default_groups[]= { "mysqladmin","client",0 };
 
-static my_bool
+my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
 {
@@ -243,6 +233,8 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     opt_count_iterations= 1;
     break;
   case 'p':
+    if (argument == disabled_my_option)
+      argument= (char*) "";			// Don't require password
     if (argument)
     {
       char *start=argument;
@@ -290,15 +282,13 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     charsets_dir = argument;
 #endif
     break;
+  case 'O':
+    WARN_DEPRECATED(VER_CELOSIA, "--set-variable", "--variable-name=value");
+    break;
   case OPT_MYSQL_PROTOCOL:
-  {
-    if ((opt_protocol= find_type(argument, &sql_protocol_typelib,0)) <= 0)
-    {
-      fprintf(stderr, "Unknown option to protocol: %s\n", argument);
-      exit(1);
-    }
+    opt_protocol= find_type_or_exit(argument, &sql_protocol_typelib,
+                                    opt->name);
     break;
-  }
   }
   if (error)
   {
@@ -324,6 +314,10 @@ int main(int argc,char *argv[])
     free_defaults(save_argv);
     exit(ho_error);
   }
+  if (debug_info_flag)
+    my_end_arg= MY_CHECK_ERROR | MY_GIVE_INFO;
+  if (debug_check_flag)
+    my_end_arg= MY_CHECK_ERROR;
 
   if (argc == 0)
   {
@@ -359,6 +353,8 @@ int main(int argc,char *argv[])
 #endif
   if (default_charset)
     mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, default_charset);
+  error_flags= (myf)(opt_nobeep ? 0 : ME_BELL);
+
   if (sql_connect(&mysql, option_wait))
   {
     /*
@@ -450,8 +446,6 @@ int main(int argc,char *argv[])
 	sleep(interval);
 	if (new_line)
 	  puts("");
-	if (opt_count_iterations)
-	  nr_iterations--;
       }
       else
         break;                                  /* no --sleep, done looping */
@@ -465,13 +459,13 @@ int main(int argc,char *argv[])
   my_free(shared_memory_base_name,MYF(MY_ALLOW_ZERO_PTR));
 #endif
   free_defaults(save_argv);
-  my_end(0);
+  my_end(my_end_arg);
   exit(error ? 1 : 0);
   return 0;
 }
 
 
-static sig_handler endprog(int signal_number __attribute__((unused)))
+sig_handler endprog(int signal_number __attribute__((unused)))
 {
   interrupted=1;
 }
@@ -513,7 +507,7 @@ static my_bool sql_connect(MYSQL *mysql, uint wait)
 	if (!host)
 	  host= (char*) LOCAL_HOST;
 	my_printf_error(0,"connect to server at '%s' failed\nerror: '%s'",
-			MYF(ME_BELL), host, mysql_error(mysql));
+			error_flags, host, mysql_error(mysql));
 	if (mysql_errno(mysql) == CR_CONNECTION_ERROR)
 	{
 	  fprintf(stderr,
@@ -602,14 +596,14 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       char buff[FN_REFLEN+20];
       if (argc < 2)
       {
-	my_printf_error(0,"Too few arguments to create",MYF(ME_BELL));
+	my_printf_error(0, "Too few arguments to create", error_flags);
 	return 1;
       }
       sprintf(buff,"create database `%.*s`",FN_REFLEN,argv[1]);
       if (mysql_query(mysql,buff))
       {
 	my_printf_error(0,"CREATE DATABASE failed; error: '%-.200s'",
-			MYF(ME_BELL), mysql_error(mysql));
+			error_flags, mysql_error(mysql));
 	return -1;
       }
       argc--; argv++;
@@ -619,7 +613,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     {
       if (argc < 2)
       {
-	my_printf_error(0,"Too few arguments to drop",MYF(ME_BELL));
+	my_printf_error(0, "Too few arguments to drop", error_flags);
 	return 1;
       }
       if (drop_db(mysql,argv[1]))
@@ -644,7 +638,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 
       if (mysql_shutdown(mysql, SHUTDOWN_DEFAULT))
       {
-	my_printf_error(0,"shutdown failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "shutdown failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -664,7 +658,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_RELOAD:
       if (mysql_query(mysql,"flush privileges"))
       {
-	my_printf_error(0,"reload failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "reload failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -675,7 +669,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 				 REFRESH_READ_LOCK | REFRESH_SLAVE |
 				 REFRESH_MASTER)))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -683,7 +677,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_FLUSH_THREADS:
       if (mysql_refresh(mysql,(uint) REFRESH_THREADS))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -691,7 +685,8 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_VER:
       new_line=1;
       print_version();
-      puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2000, 2011"));
+      puts("Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.");
+      puts("This software comes with ABSOLUTELY NO WARRANTY. This is free software,\nand you are welcome to modify and redistribute it under the GPL license\n");
       printf("Server version\t\t%s\n", mysql_get_server_info(mysql));
       printf("Protocol version\t%d\n", mysql_get_proto_info(mysql));
       printf("Connection\t\t%s\n",mysql_get_host_info(mysql));
@@ -726,7 +721,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 			      "show processlist")) ||
 	  !(result = mysql_store_result(mysql)))
       {
-	my_printf_error(0,"process list failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "process list failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -749,16 +744,22 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 	char *pos;
 	if (argc < 2)
 	{
-	  my_printf_error(0,"Too few arguments to 'kill'",MYF(ME_BELL));
+	  my_printf_error(0, "Too few arguments to 'kill'", error_flags);
 	  return 1;
 	}
 	pos=argv[1];
 	for (;;)
 	{
-	  if (mysql_kill(mysql,(ulong) atol(pos)))
+          /* We don't use mysql_kill(), since it only handles 32-bit IDs. */
+          char buff[26], *out; /* "KILL " + max 20 digs + NUL */
+          out= strxmov(buff, "KILL ", NullS);
+          ullstr(strtoull(pos, NULL, 0), out);
+
+          if (mysql_query(mysql, buff))
 	  {
-	    my_printf_error(0,"kill failed on %ld; error: '%s'",MYF(ME_BELL),
-			    atol(pos), mysql_error(mysql));
+            /* out still points to just the number */
+	    my_printf_error(0, "kill failed on %s; error: '%s'", error_flags,
+			    out, mysql_error(mysql));
 	    error=1;
 	  }
 	  if (!(pos=strchr(pos,',')))
@@ -773,7 +774,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_DEBUG:
       if (mysql_dump_debug_info(mysql))
       {
-	my_printf_error(0,"debug failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "debug failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -787,7 +788,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       if (mysql_query(mysql,"show /*!40003 GLOBAL */ variables") ||
 	  !(res=mysql_store_result(mysql)))
       {
-	my_printf_error(0,"unable to show variables; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "unable to show variables; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -809,10 +810,13 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       if (mysql_query(mysql, "show /*!50002 GLOBAL */ status") ||
 	  !(res = mysql_store_result(mysql)))
       {
-	my_printf_error(0, "unable to show status; error: '%s'", MYF(ME_BELL),
+	my_printf_error(0, "unable to show status; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
+
+      DBUG_ASSERT(mysql_num_rows(res) < MAX_MYSQL_VAR);
+
       if (!opt_vertical)
 	print_header(res);
       else
@@ -859,7 +863,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     {
       if (mysql_refresh(mysql,REFRESH_LOG))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -869,7 +873,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     {
       if (mysql_query(mysql,"flush hosts"))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -879,7 +883,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     {
       if (mysql_query(mysql,"flush tables"))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -889,7 +893,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     {
       if (mysql_query(mysql,"flush status"))
       {
-	my_printf_error(0,"refresh failed; error: '%s'",MYF(ME_BELL),
+	my_printf_error(0, "refresh failed; error: '%s'", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -906,7 +910,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 
       if (argc < 2)
       {
-	my_printf_error(0,"Too few arguments to change password",MYF(ME_BELL));
+	my_printf_error(0, "Too few arguments to change password", error_flags);
 	return 1;
       }
       if (argv[1][0])
@@ -929,7 +933,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
           if (mysql_query(mysql, "SHOW VARIABLES LIKE 'old_passwords'"))
           {
             my_printf_error(0, "Could not determine old_passwords setting from server; error: '%s'",
-                	    MYF(ME_BELL),mysql_error(mysql));
+                	    error_flags, mysql_error(mysql));
             return -1;
           }
           else
@@ -940,7 +944,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
               my_printf_error(0,
                               "Could not get old_passwords setting from "
                               "server; error: '%s'",
-        		      MYF(ME_BELL),mysql_error(mysql));
+        		      error_flags, mysql_error(mysql));
               return -1;
             }
             if (!mysql_num_rows(res))
@@ -965,7 +969,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       if (mysql_query(mysql,"set sql_log_off=1"))
       {
 	my_printf_error(0, "Can't turn off logging; error: '%s'",
-			MYF(ME_BELL),mysql_error(mysql));
+			error_flags, mysql_error(mysql));
 	return -1;
       }
       if (mysql_query(mysql,buff))
@@ -973,7 +977,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 	if (mysql_errno(mysql)!=1290)
 	{
 	  my_printf_error(0,"unable to change password; error: '%s'",
-			  MYF(ME_BELL),mysql_error(mysql));
+			  error_flags, mysql_error(mysql));
 	  return -1;
 	}
 	else
@@ -987,7 +991,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 			  " with grant tables disabled (was started with"
 			  " --skip-grant-tables).\n"
 			  "Use: \"mysqladmin flush-privileges password '*'\""
-			  " instead", MYF(ME_BELL));
+			  " instead", error_flags);
 	  return -1;
 	}
       }
@@ -998,7 +1002,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_START_SLAVE:
       if (mysql_query(mysql, "START SLAVE"))
       {
-	my_printf_error(0, "Error starting slave: %s", MYF(ME_BELL),
+	my_printf_error(0, "Error starting slave: %s", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
@@ -1008,7 +1012,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     case ADMIN_STOP_SLAVE:
       if (mysql_query(mysql, "STOP SLAVE"))
       {
-	  my_printf_error(0, "Error stopping slave: %s", MYF(ME_BELL),
+	  my_printf_error(0, "Error stopping slave: %s", error_flags,
 			  mysql_error(mysql));
 	  return -1;
       }
@@ -1034,32 +1038,14 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 	else
 	{
 	  my_printf_error(0,"mysqld doesn't answer to ping, error: '%s'",
-			  MYF(ME_BELL),mysql_error(mysql));
+			  error_flags, mysql_error(mysql));
 	  return -1;
 	}
       }
       mysql->reconnect=1;	/* Automatic reconnect is default */
       break;
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-    case ADMIN_NDB_MGM:
-    {
-      if (argc < 2)
-      {
-	my_printf_error(0,"Too few arguments to ndb-mgm",MYF(ME_BELL));
-	return 1;
-      }
-      {
-        Ndb_mgmclient_handle cmd=
-	  ndb_mgmclient_handle_create(opt_ndb_connectstring);
-        ndb_mgmclient_execute(cmd, --argc, ++argv);
-	ndb_mgmclient_handle_destroy(cmd);
-      }
-      argc= 0;
-    }
-    break;
-#endif
     default:
-      my_printf_error(0,"Unknown command: '%-.60s'",MYF(ME_BELL),argv[0]);
+      my_printf_error(0, "Unknown command: '%-.60s'", error_flags, argv[0]);
       return 1;
     }
   }
@@ -1079,7 +1065,8 @@ static void print_version(void)
 static void usage(void)
 {
   print_version();
-  puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2000, 2011"));
+  puts("Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.");
+  puts("This software comes with ABSOLUTELY NO WARRANTY. This is free software,\nand you are welcome to modify and redistribute it under the GPL license\n");
   puts("Administration program for the mysqld daemon.");
   printf("Usage: %s [OPTIONS] command command....\n", my_progname);
   my_print_help(my_long_options);
@@ -1138,7 +1125,7 @@ static int drop_db(MYSQL *mysql, const char *db)
   sprintf(name_buff,"drop database `%.*s`",FN_REFLEN,db);
   if (mysql_query(mysql,name_buff))
   {
-    my_printf_error(0,"DROP DATABASE %s failed;\nerror: '%s'",MYF(ME_BELL),
+    my_printf_error(0, "DROP DATABASE %s failed;\nerror: '%s'", error_flags,
 		    db,mysql_error(mysql));
     return 1;
   }
@@ -1363,7 +1350,7 @@ static my_bool get_pidfile(MYSQL *mysql, char *pidfile)
 
   if (mysql_query(mysql, "SHOW VARIABLES LIKE 'pid_file'"))
   {
-    my_printf_error(0,"query failed; error: '%s'",MYF(ME_BELL),
+    my_printf_error(0, "query failed; error: '%s'", error_flags,
 		    mysql_error(mysql));
   }
   result = mysql_store_result(mysql);
