@@ -1,12 +1,12 @@
-/* Copyright (c) 2001-2004 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (C) 2000-2004 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.
 
    There are special exceptions to the terms and conditions of the GPL as it
-   is applied to this software.
+   is applied to this software. View the full text of the exception in file
+   EXCEPTIONS-CLIENT in the directory of this software distribution.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include <my_global.h>
 #if defined(THREAD)
@@ -35,9 +35,7 @@
 #include <signal.h>
 #include <errno.h>
 
-#if defined(OS2)
-#  include <sys/un.h>
-#elif defined(__NETWARE__)
+#if defined(__NETWARE__)
 #include <netdb.h>
 #include <sys/select.h>
 #include <sys/utsname.h>
@@ -159,8 +157,8 @@ MYSQL_MANAGER*  STDCALL mysql_manager_connect(MYSQL_MANAGER* con,
     goto err;
   }
   sprintf(msg_buf,"%-.16s %-.16s\n",user,passwd);
-  msg_len= (uint) strlen(msg_buf);
-  if (my_net_write(&con->net,msg_buf,msg_len) || net_flush(&con->net))
+  msg_len=strlen(msg_buf);
+  if (my_net_write(&con->net,(uchar*) msg_buf,msg_len) || net_flush(&con->net))
   {
     con->last_errno=con->net.last_errno;
     strmov(con->last_error,"Write error on socket");
@@ -208,10 +206,10 @@ void STDCALL mysql_manager_close(MYSQL_MANAGER* con)
     allocated in my_multimalloc() along with con->host, freeing
     con->hosts frees the whole block
   */
-  my_free((gptr)con->host,MYF(MY_ALLOW_ZERO_PTR));
+  my_free((uchar*)con->host,MYF(MY_ALLOW_ZERO_PTR));
   net_end(&con->net);
   if (con->free_me)
-    my_free((gptr)con,MYF(0));
+    my_free((uchar*)con,MYF(0));
 }
 
 
@@ -219,8 +217,8 @@ int STDCALL mysql_manager_command(MYSQL_MANAGER* con,const char* cmd,
 				  int cmd_len)
 {
   if (!cmd_len)
-    cmd_len= (uint) strlen(cmd);
-  if (my_net_write(&con->net,(char*)cmd,cmd_len) || net_flush(&con->net))
+    cmd_len=strlen(cmd);
+  if (my_net_write(&con->net,(const uchar*)cmd,cmd_len) || net_flush(&con->net))
   {
     con->last_errno=errno;
     strmov(con->last_error,"Write error on socket");
