@@ -1,4 +1,4 @@
-/* Copyright (c) 2001, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (C) 2000 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "mysys_priv.h"
 #include "mysys_err.h"
@@ -84,20 +84,12 @@ int my_symlink(const char *content, const char *linkname, myf MyFlags)
     if (MyFlags & MY_WME)
       my_error(EE_CANT_SYMLINK, MYF(0), linkname, content, errno);
   }
+  else if ((MyFlags & MY_SYNC_DIR) && my_sync_dir_by_file(linkname, MyFlags))
+    result= -1;
   DBUG_RETURN(result);
 #endif /* HAVE_READLINK */
 }
 
-/*
-  Resolve all symbolic links in path
-  'to' may be equal to 'filename'
-
-  Because purify gives a lot of UMR errors when using realpath(),
-  this code is disabled when using purify.
-
-  If MY_RESOLVE_LINK is given, only do realpath if the file is a link.
-*/
-
 #if defined(SCO)
 #define BUFF_LEN 4097
 #elif defined(MAXPATHLEN)
@@ -122,10 +114,15 @@ int my_is_symlink(const char *filename __attribute__((unused)))
 }
 
 
+/*
+  Resolve all symbolic links in path
+  'to' may be equal to 'filename'
+*/
+
 int my_realpath(char *to, const char *filename,
 		myf MyFlags __attribute__((unused)))
 {
-#if defined(HAVE_REALPATH) && !defined(HAVE_purify) && !defined(HAVE_BROKEN_REALPATH)
+#if defined(HAVE_REALPATH) && !defined(HAVE_BROKEN_REALPATH)
   int result=0;
   char buff[BUFF_LEN];
   char *ptr;
@@ -149,23 +146,8 @@ int my_realpath(char *to, const char *filename,
     result= -1;
   }
   DBUG_RETURN(result);
-#elif defined(_WIN32)
-  int ret= GetFullPathName(filename,FN_REFLEN, to, NULL);
-  if (ret == 0 || ret > FN_REFLEN)
-  {
-    my_errno= (ret > FN_REFLEN) ? ENAMETOOLONG : GetLastError();
-    if (MyFlags & MY_WME)
-      my_error(EE_REALPATH, MYF(0), filename, my_errno);
-    /* 
-      GetFullPathName didn't work : use my_load_path() which is a poor 
-      substitute original name but will at least be able to resolve 
-      paths that starts with '.'.
-    */  
-    my_load_path(to, filename, NullS);
-    return -1;
-  }
 #else
   my_load_path(to, filename, NullS);
-#endif  
   return 0;
+#endif
 }
