@@ -1,5 +1,4 @@
-/* Copyright (c) 2000, 2001, 2004-2007 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (C) 2000 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include "mysys_priv.h"
 #include "mysys_err.h"
@@ -21,14 +20,14 @@
 
 	/* Write a chunk of bytes to a file */
 
-uint my_write(int Filedes, const byte *Buffer, uint Count, myf MyFlags)
+size_t my_write(int Filedes, const uchar *Buffer, size_t Count, myf MyFlags)
 {
-  uint writenbytes,errors;
-  ulong written;
+  size_t writenbytes, written;
+  uint errors;
   DBUG_ENTER("my_write");
-  DBUG_PRINT("my",("Fd: %d  Buffer: 0x%lx  Count: %d  MyFlags: %d",
-		   Filedes, (long) Buffer, Count, MyFlags));
-  errors=0; written=0L;
+  DBUG_PRINT("my",("Fd: %d  Buffer: 0x%lx  Count: %lu  MyFlags: %d",
+		   Filedes, (long) Buffer, (ulong) Count, MyFlags));
+  errors=0; written=0;
 
   /* The behavior of write(fd, buf, 0) is not portable */
   if (unlikely(!Count))
@@ -36,17 +35,17 @@ uint my_write(int Filedes, const byte *Buffer, uint Count, myf MyFlags)
   
   for (;;)
   {
-    if ((writenbytes = (uint) write(Filedes, Buffer, Count)) == Count)
+    if ((writenbytes= write(Filedes, Buffer, Count)) == Count)
       break;
-    if ((int) writenbytes != -1)
+    if (writenbytes != (size_t) -1)
     {						/* Safeguard */
       written+=writenbytes;
       Buffer+=writenbytes;
       Count-=writenbytes;
     }
     my_errno=errno;
-    DBUG_PRINT("error",("Write only %d bytes, error: %d",
-			writenbytes,my_errno));
+    DBUG_PRINT("error",("Write only %ld bytes, error: %d",
+			(long) writenbytes, my_errno));
 #ifndef NO_BACKGROUND
 #ifdef THREAD
     if (my_thread_var->abort)
@@ -60,12 +59,12 @@ uint my_write(int Filedes, const byte *Buffer, uint Count, myf MyFlags)
       continue;
     }
 
-    if ((writenbytes == 0 || (int) writenbytes == -1))
+    if ((writenbytes == 0 || writenbytes == (size_t) -1))
     {
       if (my_errno == EINTR)
       {
-        DBUG_PRINT("debug", ("my_write() was interrupted and returned %d",
-                             (int) writenbytes));
+        DBUG_PRINT("debug", ("my_write() was interrupted and returned %ld",
+                             (long) writenbytes));
         continue;                               /* Interrupted */
       }
 
