@@ -1,5 +1,4 @@
-/* Copyright (c) 2004-2007 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (C) 2004-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #if defined(__GNUC__) && defined(USE_PRAGMA_IMPLEMENTATION)
 #pragma implementation
@@ -20,84 +19,34 @@
 
 #include "mysql_connection.h"
 
-#include "priv.h"
-#include "mysql_manager_error.h"
-#include "mysqld_error.h"
-#include "thread_registry.h"
-#include "log.h"
-#include "user_map.h"
-#include "protocol.h"
-#include "messages.h"
-#include "command.h"
-#include "parse.h"
-
-#include <mysql.h>
-#include <violite.h>
-#include <mysql_com.h>
 #include <m_string.h>
+#include <m_string.h>
+#include <my_global.h>
+#include <mysql.h>
 #include <my_sys.h>
+#include <violite.h>
+
+#include "command.h"
+#include "log.h"
+#include "messages.h"
+#include "mysqld_error.h"
+#include "mysql_manager_error.h"
+#include "parse.h"
+#include "priv.h"
+#include "protocol.h"
+#include "thread_registry.h"
+#include "user_map.h"
 
 
-Mysql_connection_thread_args::Mysql_connection_thread_args(
-                             struct st_vio *vio_arg,
-                             Thread_registry &thread_registry_arg,
-                             const User_map &user_map_arg,
-                             ulong connection_id_arg,
-                             Instance_map &instance_map_arg) :
-    vio(vio_arg)
-    ,thread_registry(thread_registry_arg)
-    ,user_map(user_map_arg)
-    ,connection_id(connection_id_arg)
-    ,instance_map(instance_map_arg)
-  {}
-
-/*
-  MySQL connection - handle one connection with mysql command line client
-  See also comments in mysqlmanager.cc to picture general Instance Manager
-  architecture.
-  We use conventional technique to work with classes without exceptions:
-  class acquires all vital resource in init(); Thus if init() succeed, 
-  a user must call cleanup(). All other methods are valid only between
-  init() and cleanup().
-*/
-
-class Mysql_connection_thread: public Mysql_connection_thread_args
+Mysql_connection::Mysql_connection(Thread_registry *thread_registry_arg,
+                                   User_map *user_map_arg,
+                                   struct st_vio *vio_arg, ulong
+                                   connection_id_arg)
+  :vio(vio_arg),
+  connection_id(connection_id_arg),
+  thread_registry(thread_registry_arg),
+  user_map(user_map_arg)
 {
-public:
-  Mysql_connection_thread(const Mysql_connection_thread_args &args);
-
-  int init();
-  void cleanup();
-
-  void run();
-
-  ~Mysql_connection_thread();
-private:
-  Thread_info thread_info;
-  NET net;
-  struct rand_struct rand_st;
-  char scramble[SCRAMBLE_LENGTH + 1];
-  uint status;
-  ulong client_capabilities;
-private:
-  /* Names are conventionally the same as in mysqld */
-  int check_connection();
-  int do_command();
-  int dispatch_command(enum enum_server_command command,
-                       const char *text, uint len);
-};
-
-
-Mysql_connection_thread::Mysql_connection_thread(
-                                   const Mysql_connection_thread_args &args) :
-  Mysql_connection_thread_args(args.vio,
-                               args.thread_registry,
-                               args.user_map,
-                               args.connection_id,
-                               args.instance_map)
-  ,thread_info(pthread_self())
-{
-  thread_registry.register_thread(&thread_info);
 }
 
 
@@ -139,83 +88,87 @@ C_MODE_END
   This function is complementary to cleanup().
 */
 
-int Mysql_connection_thread::init()
+bool Mysql_connection::init()
 {
   /* Allocate buffers for network I/O */
   if (my_net_init(&net, vio))
-    return 1;
+    return TRUE;
+
   net.return_status= &status;
+
   /* Initialize random number generator */
   {
     ulong seed1= (ulong) &rand_st + rand();
-    ulong seed2= rand() + (ulong) time(0);
+    ulong seed2= (ulong) rand() + (ulong) time(0);
     randominit(&rand_st, seed1, seed2);
   }
+
   /* Fill scramble - server's random message used for handshake */
   create_random_string(scramble, SCRAMBLE_LENGTH, &rand_st);
+
   /* We don't support transactions, every query is atomic */
   status= SERVER_STATUS_AUTOCOMMIT;
-  return 0;
+
+  thread_registry->register_thread(&thread_info);
+
+  return FALSE;
 }
 
 
-void Mysql_connection_thread::cleanup()
+void Mysql_connection::cleanup()
 {
   net_end(&net);
+  thread_registry->unregister_thread(&thread_info);
 }
 
 
-Mysql_connection_thread::~Mysql_connection_thread()
+Mysql_connection::~Mysql_connection()
 {
   /* vio_delete closes the socket if necessary */
   vio_delete(vio);
-  thread_registry.unregister_thread(&thread_info);
 }
 
 
-void Mysql_connection_thread::run()
+void Mysql_connection::main()
 {
-  log_info("accepted mysql connection %lu", connection_id);
-
-  my_thread_init();
+  log_info("Connection %lu: accepted.", (unsigned long) connection_id);
 
   if (check_connection())
   {
-    my_thread_end();
+    log_info("Connection %lu: failed to authorize the user.",
+            (unsigned long) connection_id);
+
     return;
   }
 
-  log_info("connection %lu is checked successfully", connection_id);
+  log_info("Connection %lu: the user was authorized successfully.",
+           (unsigned long) connection_id);
 
   vio_keepalive(vio, TRUE);
 
-  while (!net.error && net.vio && !thread_registry.is_shutdown())
+  while (!net.error && net.vio && !thread_registry->is_shutdown())
   {
     if (do_command())
       break;
   }
-
-  my_thread_end();
 }
 
 
-int Mysql_connection_thread::check_connection()
+int Mysql_connection::check_connection()
 {
   ulong pkt_len=0;                              // to hold client reply length
-  /* maximum size of the version string */
-  enum { MAX_VERSION_LENGTH= 80 };
 
   /* buffer for the first packet */             /* packet contains: */
-  char buff[MAX_VERSION_LENGTH + 1 +            // server version, 0-ended
-            4 +                                 // connection id
-            SCRAMBLE_LENGTH + 2 +               // scramble (in 2 pieces)
-            18];                                // server variables: flags,
+  uchar buff[MAX_VERSION_LENGTH + 1 +            // server version, 0-ended
+             4 +                                 // connection id
+             SCRAMBLE_LENGTH + 2 +               // scramble (in 2 pieces)
+             18];                                // server variables: flags,
                                                 // charset number, status,
-  char *pos= buff;
+  uchar *pos= buff;
   ulong server_flags;
 
-  memcpy(pos, mysqlmanager_version, mysqlmanager_version_length + 1);
-  pos+= mysqlmanager_version_length + 1;
+  memcpy(pos, mysqlmanager_version.str, mysqlmanager_version.length + 1);
+  pos+= mysqlmanager_version.length + 1;
 
   int4store((uchar*) pos, connection_id);
   pos+= 4;
@@ -254,7 +207,8 @@ int Mysql_connection_thread::check_connection()
 
   /* write connection message and read reply */
   enum { MIN_HANDSHAKE_SIZE= 2 };
-  if (net_write_command(&net, protocol_version, "", 0, buff, (uint) (pos - buff)) ||
+  if (net_write_command(&net, protocol_version, (uchar*) "", 0,
+                        buff, pos - buff) ||
      (pkt_len= my_net_read(&net)) == packet_error ||
       pkt_len < MIN_HANDSHAKE_SIZE)
   {
@@ -270,25 +224,27 @@ int Mysql_connection_thread::check_connection()
   }
   client_capabilities|= ((ulong) uint2korr(net.read_pos + 2)) << 16;
 
-  pos= (char*) net.read_pos + 32;
+  pos= net.read_pos + 32;
 
   /* At least one byte for username and one byte for password */
-  if (pos >= (char*) net.read_pos + pkt_len + 2)
+  if (pos >= net.read_pos + pkt_len + 2)
   {
     /*TODO add user and password handling in error messages*/
     net_send_error(&net, ER_HANDSHAKE_ERROR);
     return 1;
   }
 
-  const char *user= pos;
+  const char *user= (char*) pos;
   const char *password= strend(user)+1;
   ulong password_len= *password++;
+  LEX_STRING user_name= { (char *) user, password - user - 2 };
+
   if (password_len != SCRAMBLE_LENGTH)
   {
     net_send_error(&net, ER_ACCESS_DENIED_ERROR);
     return 1;
   }
-  if (user_map.authenticate(user, (uint) (password - user - 2), password, scramble))
+  if (user_map->authenticate(&user_name, password, scramble))
   {
     net_send_error(&net, ER_ACCESS_DENIED_ERROR);
     return 1;
@@ -298,7 +254,7 @@ int Mysql_connection_thread::check_connection()
 }
 
 
-int Mysql_connection_thread::do_command()
+int Mysql_connection::do_command()
 {
   char *packet;
   ulong packet_length;
@@ -311,7 +267,7 @@ int Mysql_connection_thread::do_command()
     /* Check if we can continue without closing the connection */
     if (net.error != 3) // what is 3 - find out
       return 1;
-    if (thread_registry.is_shutdown())
+    if (thread_registry->is_shutdown())
       return 1;
     net_send_error(&net, net.last_errno);
     net.error= 0;
@@ -319,78 +275,102 @@ int Mysql_connection_thread::do_command()
   }
   else
   {
-    if (thread_registry.is_shutdown())
+    if (thread_registry->is_shutdown())
       return 1;
     packet= (char*) net.read_pos;
     enum enum_server_command command= (enum enum_server_command)
                                       (uchar) *packet;
-    log_info("connection %lu: packet_length=%lu, command=%d", 
-             connection_id, packet_length, command);
-    return dispatch_command(command, packet + 1, packet_length - 1);
+    log_info("Connection %lu: received packet (length: %lu; command: %d).",
+             (unsigned long) connection_id,
+             (unsigned long) packet_length,
+             (int) command);
+
+    return dispatch_command(command, packet + 1);
   }
 }
 
-int Mysql_connection_thread::dispatch_command(enum enum_server_command command,
-                                              const char *packet, uint len)
+int Mysql_connection::dispatch_command(enum enum_server_command command,
+                                       const char *packet)
 {
   switch (command) {
   case COM_QUIT:                                // client exit
-    log_info("query for connection %lu received quit command", connection_id);
+    log_info("Connection %lu: received QUIT command.",
+             (unsigned long) connection_id);
     return 1;
+
   case COM_PING:
-    log_info("query for connection %lu received ping command", connection_id);
+    log_info("Connection %lu: received PING command.",
+             (unsigned long) connection_id);
     net_send_ok(&net, connection_id, NULL);
-    break;
+    return 0;
+
   case COM_QUERY:
   {
-    log_info("query for connection %lu : ----\n%s\n-------------------------",
-	     connection_id,packet);
-    if (Command *com= parse_command(&instance_map, packet))
+    log_info("Connection %lu: received QUERY command: '%s'.",
+	     (unsigned long) connection_id,
+             (const char *) packet);
+
+    if (Command *com= parse_command(packet))
     {
       int res= 0;
-      log_info("query for connection %lu successefully parsed",connection_id);
+
+      log_info("Connection %lu: query parsed successfully.",
+               (unsigned long) connection_id);
+
       res= com->execute(&net, connection_id);
       delete com;
       if (!res)
-        log_info("query for connection %lu executed ok",connection_id);
+      {
+        log_info("Connection %lu: query executed successfully",
+                 (unsigned long) connection_id);
+      }
       else
       {
-        log_info("query for connection %lu executed err=%d",connection_id,res);
+        log_info("Connection %lu: can not execute query (error: %d).",
+                 (unsigned long) connection_id,
+                 (int) res);
+
         net_send_error(&net, res);
-        return 0;
       }
     }
     else
     {
+      log_error("Connection %lu: can not parse query: out ot resources.",
+                (unsigned long) connection_id);
+
       net_send_error(&net,ER_OUT_OF_RESOURCES);
-      return 0;
     }
-    break;
+
+    return 0;
   }
+
   default:
-    log_info("query for connection %lu received unknown command",connection_id);
+    log_info("Connection %lu: received unsupported command (%d).",
+              (unsigned long) connection_id,
+              (int) command);
+
     net_send_error(&net, ER_UNKNOWN_COM_ERROR);
-    break;
+    return 0;
   }
-  return 0;
+
+  return 0; /* Just to make compiler happy. */
 }
 
 
-pthread_handler_t mysql_connection(void *arg)
+void Mysql_connection::run()
 {
-  Mysql_connection_thread_args *args= (Mysql_connection_thread_args *) arg;
-  Mysql_connection_thread mysql_connection_thread(*args);
-  delete args;
-  if (mysql_connection_thread.init())
-    log_info("mysql_connection(): error initializing thread");
+  if (init())
+    log_error("Connection %lu: can not init handler.",
+              (unsigned long) connection_id);
   else
   {
-    mysql_connection_thread.run();
-    mysql_connection_thread.cleanup();
+    main();
+    cleanup();
   }
-  return 0;
+
+  delete this;
 }
 
 /*
-  vim: fdm=marker 
+  vim: fdm=marker
 */
