@@ -1,6 +1,5 @@
 #!/usr/bin/perl
-# Copyright (c) 2000-2003, 2006 MySQL AB, 2009 Sun Microsystems, Inc.
-# Use is subject to license terms.
+# Copyright (C) 2000-2003 MySQL AB
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +13,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the Free
-# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-# MA 02110-1301, USA
+# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+# MA 02111-1307, USA
 #
 # Test of creating a simple table and inserting $record_count records in it,
 # $opt_loop_count rows in order, $opt_loop_count rows in reverse order and
@@ -999,7 +998,7 @@ for ($i=1 ; $i < $opt_loop_count*3 ; $i+=3)
 {
   $sth = $dbh->do("update bench1 set dummy1='updated' where id=$i") or die $DBI::errstr;
   $end_time=new Benchmark;
-  last if ($estimated=predict_query_time($loop_time,$end_time,\$i,$tests,
+  last if ($estimated=predict_query_time($loop_time,$end_time,\$i,($i-1)/3,
 					 $opt_loop_count));
 }
 if ($estimated)
