@@ -23,6 +23,11 @@
 
 #include <stddef.h>
 
+/**
+  @defgroup Stored_Routines Stored Routines
+  @ingroup Runtime_Environment
+  @{
+*/
 // Values for the type enum. This reflects the order of the enum declaration
 // in the CREATE TABLE command.
 #define TYPE_ENUM_FUNCTION  1
@@ -45,6 +50,58 @@ class sp_instr_jump_if_not;
 struct sp_cond_type;
 struct sp_variable;
 
+/*************************************************************************/
+
+/**
+  Stored_program_creation_ctx -- base class for creation context of stored
+  programs (stored routines, triggers, events).
+*/
+
+class Stored_program_creation_ctx :public Default_object_creation_ctx
+{
+public:
+  CHARSET_INFO *get_db_cl()
+  {
+    return m_db_cl;
+  }
+
+public:
+  virtual Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) = 0;
+
+protected:
+  Stored_program_creation_ctx(THD *thd)
+    : Default_object_creation_ctx(thd),
+      m_db_cl(thd->variables.collation_database)
+  { }
+
+  Stored_program_creation_ctx(CHARSET_INFO *client_cs,
+                              CHARSET_INFO *connection_cl,
+                              CHARSET_INFO *db_cl)
+    : Default_object_creation_ctx(client_cs, connection_cl),
+      m_db_cl(db_cl)
+  { }
+
+protected:
+  virtual void change_env(THD *thd) const
+  {
+    thd->variables.collation_database= m_db_cl;
+
+    Default_object_creation_ctx::change_env(thd);
+  }
+
+protected:
+  /**
+    db_cl stores the value of the database collation. Both character set
+    and collation attributes are used.
+
+    Database collation is included into the context because it defines the
+    default collation for stored-program variables.
+  */
+  CHARSET_INFO *m_db_cl;
+};
+
+/*************************************************************************/
+
 class sp_name : public Sql_alloc
 {
 public:
@@ -52,7 +109,7 @@ public:
   LEX_STRING m_db;
   LEX_STRING m_name;
   LEX_STRING m_qname;
-  /*
+  /**
     Key representing routine in the set of stored routines used by statement.
     Consists of 1-byte routine type and m_qname (which usually refences to
     same buffer). Note that one must complete initialization of the key by
@@ -68,7 +125,7 @@ public:
     m_qname.length= m_sroutines_key.length= 0;
   }
 
-  /*
+  /**
     Creates temporary sp_name object from key, used mainly
     for SP-cache lookups.
   */
@@ -88,16 +145,16 @@ public:
 
 
 bool
-check_routine_name(LEX_STRING name);
+check_routine_name(LEX_STRING *ident);
 
 class sp_head :private Query_arena
 {
-  sp_head(const sp_head &);	/* Prevent use of these */
+  sp_head(const sp_head &);	/**< Prevent use of these */
   void operator=(sp_head &);
 
   MEM_ROOT main_mem_root;
 public:
-  /* Possible values of m_flags */
+  /** Possible values of m_flags */
   enum {
     HAS_RETURN= 1,              // For FUNCTIONs only: is set if has RETURN
     MULTI_RESULTS= 8,           // Is set if a procedure with SELECT(s)
@@ -106,24 +163,27 @@ public:
     HAS_SET_AUTOCOMMIT_STMT= 64,// Is set if a procedure with 'set autocommit'
     /* Is set if a procedure with COMMIT (implicit or explicit) | ROLLBACK */
     HAS_COMMIT_OR_ROLLBACK= 128,
+    LOG_SLOW_STATEMENTS= 256,   // Used by events
+    LOG_GENERAL_LOG= 512,        // Used by events
+    BINLOG_ROW_BASED_IF_MIXED= 1024,
     HAS_SQLCOM_RESET= 2048,
     HAS_SQLCOM_FLUSH= 4096
   };
 
-  /* TYPE_ENUM_FUNCTION, TYPE_ENUM_PROCEDURE or TYPE_ENUM_TRIGGER */
+  /** TYPE_ENUM_FUNCTION, TYPE_ENUM_PROCEDURE or TYPE_ENUM_TRIGGER */
   int m_type;
   uint m_flags;                 // Boolean attributes of a stored routine
 
-  create_field m_return_field_def; /* This is used for FUNCTIONs only. */
+  Create_field m_return_field_def; /**< This is used for FUNCTIONs only. */
 
-  const char *m_tmp_query;	// Temporary pointer to sub query string
+  const char *m_tmp_query;	///< Temporary pointer to sub query string
   st_sp_chistics *m_chistics;
-  ulong m_sql_mode;		// For SHOW CREATE and execution
-  LEX_STRING m_qname;		// db.name
-  bool m_explicit_name;                /**< Prepend the db name? */
+  ulong m_sql_mode;		///< For SHOW CREATE and execution
+  LEX_STRING m_qname;		///< db.name
+  bool m_explicit_name;         ///< Prepend the db name? */
   /**
     Key representing routine in the set of stored routines used by statement.
-    [routine_type]db.name\0
+    [routine_type]db.name
     @sa sp_name::m_sroutines_key
   */
   LEX_STRING m_sroutines_key;
@@ -131,25 +191,41 @@ public:
   LEX_STRING m_name;
   LEX_STRING m_params;
   LEX_STRING m_body;
+  LEX_STRING m_body_utf8;
   LEX_STRING m_defstr;
   LEX_STRING m_definer_user;
   LEX_STRING m_definer_host;
+
+private:
+  Stored_program_creation_ctx *m_creation_ctx;
+
+public:
+  inline Stored_program_creation_ctx *get_creation_ctx()
+  {
+    return m_creation_ctx;
+  }
+
+  inline void set_creation_ctx(Stored_program_creation_ctx *creation_ctx)
+  {
+    m_creation_ctx= creation_ctx->clone(mem_root);
+  }
+
   longlong m_created;
   longlong m_modified;
-  /* Recursion level of the current SP instance. The levels are numbered from 0 */
+  /** Recursion level of the current SP instance. The levels are numbered from 0 */
   ulong m_recursion_level;
-  /*
+  /**
     A list of diferent recursion level instances for the same procedure.
     For every recursion level we have a sp_head instance. This instances
     connected in the list. The list ordered by increasing recursion level
     (m_recursion_level).
   */
   sp_head *m_next_cached_sp;
-  /*
+  /**
     Pointer to the first element of the above list
   */
   sp_head *m_first_instance;
-  /*
+  /**
     Pointer to the first free (non-INVOKED) routine in the list of
     cached instances for this SP. This pointer is set only for the first
     SP in the list of instences (see above m_first_cached_sp pointer).
@@ -157,12 +233,12 @@ public:
     For non-first instance value of this pointer meanless (point to itself);
   */
   sp_head *m_first_free_instance;
-  /*
+  /**
     Pointer to the last element in the list of instances of the SP.
     For non-first instance value of this pointer meanless (point to itself);
   */
   sp_head *m_last_cached_sp;
-  /*
+  /**
     Set containing names of stored routines used by this routine.
     Note that unlike elements of similar set for statement elements of this
     set are not linked in one list. Because of this we are able save memory
@@ -173,8 +249,11 @@ public:
   // Pointers set during parsing
   const char *m_param_begin;
   const char *m_param_end;
+
+private:
   const char *m_body_begin;
 
+public:
   /*
     Security context for stored routine which should be run under
     definer privileges.
@@ -189,30 +268,36 @@ public:
 
   sp_head();
 
-  // Initialize after we have reset mem_root
+  /// Initialize after we have reset mem_root
   void
   init(LEX *lex);
 
-  /* Copy sp name from parser. */
+  /** Copy sp name from parser. */
   void
   init_sp_name(THD *thd, sp_name *spname);
 
-  // Initialize strings after parsing header
+  /** Set the body-definition start position. */
   void
-  init_strings(THD *thd, LEX *lex);
+  set_body_start(THD *thd, const char *begin_ptr);
+
+  /** Set the statement-definition (body-definition) end position. */
+  void
+  set_stmt_end(THD *thd);
 
   int
   create(THD *thd);
 
   virtual ~sp_head();
 
-  // Free memory
+  /// Free memory
   void
   destroy();
 
   bool
-  execute_trigger(THD *thd, const char *db, const char *table,
-                  GRANT_INFO *grant_onfo);
+  execute_trigger(THD *thd,
+                  const LEX_STRING *db_name,
+                  const LEX_STRING *table_name,
+                  GRANT_INFO *grant_info);
 
   bool
   execute_function(THD *thd, Item **args, uint argcount, Field *return_fld);
@@ -220,11 +305,8 @@ public:
   bool
   execute_procedure(THD *thd, List<Item> *args);
 
-  int
-  show_create_procedure(THD *thd);
-
-  int
-  show_create_function(THD *thd);
+  bool
+  show_create_routine(THD *thd, int type);
 
   int
   add_instr(sp_instr *instr);
@@ -240,44 +322,52 @@ public:
   {
     sp_instr *i;
 
-    get_dynamic(&m_instr, (gptr)&i, m_instr.elements-1);
+    get_dynamic(&m_instr, (uchar*)&i, m_instr.elements-1);
     return i;
   }
 
-  // Resets lex in 'thd' and keeps a copy of the old one.
+  /*
+    Resets lex in 'thd' and keeps a copy of the old one.
+
+    @todo Conflicting comment in sp_head.cc
+  */
   bool
   reset_lex(THD *thd);
 
-  // Restores lex in 'thd' from our copy, but keeps some status from the
-  // one in 'thd', like ptr, tables, fields, etc.
-  void
+  /**
+    Restores lex in 'thd' from our copy, but keeps some status from the
+    one in 'thd', like ptr, tables, fields, etc.
+
+    @todo Conflicting comment in sp_head.cc
+  */
+  bool
   restore_lex(THD *thd);
 
-  // Put the instruction on the backpatch list, associated with the label.
+  /// Put the instruction on the backpatch list, associated with the label.
   int
   push_backpatch(sp_instr *, struct sp_label *);
 
-  // Update all instruction with this label in the backpatch list to
-  // the current position.
+  /// Update all instruction with this label in the backpatch list to
+  /// the current position.
   void
   backpatch(struct sp_label *);
 
-  // Start a new cont. backpatch level. If 'i' is NULL, the level is just incr.
+  /// Start a new cont. backpatch level. If 'i' is NULL, the level is just incr.
   int
   new_cont_backpatch(sp_instr_opt_meta *i);
 
-  // Add an instruction to the current level
+  /// Add an instruction to the current level
   int
   add_cont_backpatch(sp_instr_opt_meta *i);
 
-  // Backpatch (and pop) the current level to the current position.
+  /// Backpatch (and pop) the current level to the current position.
   void
   do_cont_backpatch();
 
   char *name(uint *lenp = 0) const
   {
     if (lenp)
-      *lenp= m_name.length;
+      *lenp= (uint) m_name.length;
     return m_name.str;
   }
 
@@ -288,7 +378,7 @@ public:
 
   bool fill_field_definition(THD *thd, LEX *lex,
                              enum enum_field_types field_type,
-                             create_field *field_def);
+                             Create_field *field_def);
 
   void set_info(longlong created, longlong modified,
 		st_sp_chistics *chistics, ulong sql_mode);
@@ -322,7 +412,7 @@ public:
     sp_instr *ip;
 
     if (i < m_instr.elements)
-      get_dynamic(&m_instr, (gptr)&ip, i);
+      get_dynamic(&m_instr, (uchar*)&ip, i);
     else
       ip= NULL;
     return ip;
@@ -333,7 +423,7 @@ public:
                                      TABLE_LIST ***query_tables_last_ptr,
                                      TABLE_LIST *belong_to_view);
 
-  /*
+  /**
     Check if this stored routine contains statements disallowed
     in a stored function or trigger, and set an appropriate error message
     if this is the case.
@@ -362,22 +452,39 @@ public:
   int show_routine_code(THD *thd);
 #endif
 
+  /*
+    This method is intended for attributes of a routine which need
+    to propagate upwards to the LEX of the caller (when a property of a
+    sp_head needs to "taint" the caller).
+  */
+  void propagate_attributes(LEX *lex)
+  {
+    /*
+      If this routine needs row-based binary logging, the entire top statement
+      too (we cannot switch from statement-based to row-based only for this
+      routine, as in statement-based the top-statement may be binlogged and
+      the substatements not).
+    */
+    if (m_flags & BINLOG_ROW_BASED_IF_MIXED)
+      lex->set_stmt_unsafe();
+  }
+
 
 private:
 
-  MEM_ROOT *m_thd_root;		// Temp. store for thd's mem_root
-  THD *m_thd;			// Set if we have reset mem_root
+  MEM_ROOT *m_thd_root;		///< Temp. store for thd's mem_root
+  THD *m_thd;			///< Set if we have reset mem_root
 
-  sp_pcontext *m_pcont;		// Parse context
-  List<LEX> m_lex;		// Temp. store for the other lex
-  DYNAMIC_ARRAY m_instr;	// The "instructions"
+  sp_pcontext *m_pcont;		///< Parse context
+  List<LEX> m_lex;		///< Temp. store for the other lex
+  DYNAMIC_ARRAY m_instr;	///< The "instructions"
   typedef struct
   {
     struct sp_label *lab;
     sp_instr *instr;
   } bp_t;
-  List<bp_t> m_backpatch;	// Instructions needing backpatching
-  /*
+  List<bp_t> m_backpatch;	///< Instructions needing backpatching
+  /**
     We need a special list for backpatching of instructions with a continue
     destination (in the case of a continue handler catching an error in
     the test), since it would otherwise interfere with the normal backpatch
@@ -385,15 +492,16 @@ private:
     which are to be patched differently.
     Since these occur in a more restricted way (always the same "level" in
     the code), we don't need the label.
-   */
+  */
   List<sp_instr_opt_meta> m_cont_backpatch;
   uint m_cont_level;            // The current cont. backpatch level
 
-  /*
+  /**
     Multi-set representing optimized list of tables to be locked by this
     routine. Does not include tables which are used by invoked routines.
 
-    Note: for prelocking-free SPs this multiset is constructed too.
+    @note
+    For prelocking-free SPs this multiset is constructed too.
     We do so because the same instance of sp_head may be called both
     in prelocked mode and in non-prelocked mode.
   */
@@ -408,7 +516,7 @@ private:
   */
   void opt_mark();
 
-  /*
+  /**
     Merge the list of tables used by query into the multi-set of tables used
     by routine.
   */
@@ -422,16 +530,16 @@ private:
 
 class sp_instr :public Query_arena, public Sql_alloc
 {
-  sp_instr(const sp_instr &);	/* Prevent use of these */
+  sp_instr(const sp_instr &);	/**< Prevent use of these */
   void operator=(sp_instr &);
 
 public:
 
   uint marked;
-  uint m_ip;			// My index
-  sp_pcontext *m_ctx;		// My parse context
+  uint m_ip;			///< My index
+  sp_pcontext *m_ctx;		///< My parse context
 
-  // Should give each a name or type code for debugging purposes?
+  /// Should give each a name or type code for debugging purposes?
   sp_instr(uint ip, sp_pcontext *ctx)
     :Query_arena(0, INITIALIZED_FOR_SP), marked(0), m_ip(ip), m_ctx(ctx)
   {}
@@ -440,21 +548,19 @@ public:
   { free_items(); }
 
 
-  /*
+  /**
     Execute this instruction
 
-    SYNOPSIS
-       execute()
-         thd        Thread handle
-         nextp  OUT index of the next instruction to execute. (For most
-                    instructions this will be the instruction following this
-                    one). Note that this parameter is undefined in case of
-                    errors, use get_cont_dest() to find the continuation
-                    instruction for CONTINUE error handlers.
-
-     RETURN
-       0      on success,
-       other  if some error occurred
+   
+    @param thd         Thread handle
+    @param[out] nextp  index of the next instruction to execute. (For most
+                       instructions this will be the instruction following this
+                       one). Note that this parameter is undefined in case of
+                       errors, use get_cont_dest() to find the continuation
+                       instruction for CONTINUE error handlers.
+   
+    @retval 0      on success, 
+    @retval other  if some error occured
   */
 
   virtual int execute(THD *thd, uint *nextp) = 0;
@@ -492,7 +598,7 @@ public:
   virtual void backpatch(uint dest, sp_pcontext *dst_ctx)
   {}
 
-  /*
+  /**
     Mark this instruction as reachable during optimization and return the
     index to the next instruction. Jump instruction will add their
     destination to the leads list.
@@ -503,7 +609,7 @@ public:
     return m_ip+1;
   }
 
-  /*
+  /**
     Short-cut jumps to jumps during optimization. This is used by the
     jump instructions' opt_mark() methods. 'start' is the starting point,
     used to prevent the mark sweep from looping for ever. Return the
@@ -514,7 +620,7 @@ public:
     return m_ip;
   }
 
-  /*
+  /**
     Inform the instruction that it has been moved during optimization.
     Most instructions will simply update its index, but jump instructions
     must also take care of their destination pointers. Forward jumps get
@@ -528,7 +634,7 @@ public:
 }; // class sp_instr : public Sql_alloc
 
 
-/*
+/**
   Auxilary class to which instructions delegate responsibility
   for handling LEX and preparations before executing statement
   or calculating complex expression.
@@ -536,13 +642,14 @@ public:
   Exist mainly to avoid having double hierarchy between instruction
   classes.
 
-  TODO: Add ability to not store LEX and do any preparations if
-        expression used is simple.
+  @todo
+    Add ability to not store LEX and do any preparations if
+    expression used is simple.
 */
 
 class sp_lex_keeper
 {
-  /* Prevent use of these */
+  /** Prevent use of these */
   sp_lex_keeper(const sp_lex_keeper &);
   void operator=(sp_lex_keeper &);
 public:
@@ -562,10 +669,12 @@ public:
     }
   }
 
-  /*
+  /**
     Prepare execution of instruction using LEX, if requested check whenever
     we have read access to tables used and open/lock them, call instruction's
     exec_core() method, perform cleanup afterwards.
+   
+    @todo Conflicting comment in sp_head.cc
   */
   int reset_lex_and_exec_core(THD *thd, uint *nextp, bool open_tables,
                               sp_instr* instr);
@@ -582,7 +691,7 @@ public:
 private:
 
   LEX *m_lex;
-  /*
+  /**
     Indicates whenever this sp_lex_keeper instance responsible
     for LEX deletion.
   */
@@ -595,13 +704,13 @@ private:
        prelocked mode itself.
   */
   
-  /*
+  /**
     List of additional tables this statement needs to lock when it
     enters/leaves prelocked mode on its own.
   */
   TABLE_LIST *prelocking_tables;
 
-  /*
+  /**
     The value m_lex->query_tables_own_last should be set to this when the
     statement enters/leaves prelocked mode on its own.
   */
@@ -609,17 +718,17 @@ private:
 };
 
 
-//
-// Call out to some prepared SQL statement.
-//
+/**
+  Call out to some prepared SQL statement.
+*/
 class sp_instr_stmt : public sp_instr
 {
-  sp_instr_stmt(const sp_instr_stmt &);	/* Prevent use of these */
+  sp_instr_stmt(const sp_instr_stmt &);	/**< Prevent use of these */
   void operator=(sp_instr_stmt &);
 
 public:
 
-  LEX_STRING m_query;		// For thd->query
+  LEX_STRING m_query;		///< For thd->query
 
   sp_instr_stmt(uint ip, sp_pcontext *ctx, LEX *lex)
     : sp_instr(ip, ctx), m_lex_keeper(lex, TRUE)
@@ -646,7 +755,7 @@ private:
 
 class sp_instr_set : public sp_instr
 {
-  sp_instr_set(const sp_instr_set &);	/* Prevent use of these */
+  sp_instr_set(const sp_instr_set &);	/**< Prevent use of these */
   void operator=(sp_instr_set &);
 
 public:
@@ -669,15 +778,15 @@ public:
 
 private:
 
-  uint m_offset;		// Frame offset
+  uint m_offset;		///< Frame offset
   Item *m_value;
-  enum enum_field_types m_type;	// The declared type
+  enum enum_field_types m_type;	///< The declared type
   sp_lex_keeper m_lex_keeper;
 
 }; // class sp_instr_set : public sp_instr
 
 
-/*
+/**
   Set NEW/OLD row field value instruction. Used in triggers.
 */
 class sp_instr_set_trigger_field : public sp_instr
@@ -711,18 +820,19 @@ private:
 }; // class sp_instr_trigger_field : public sp_instr
 
 
-/*
+/**
   An abstract class for all instructions with destinations that
   needs to be updated by the optimizer.
+
   Even if not all subclasses will use both the normal destination and
   the continuation destination, we put them both here for simplicity.
- */
+*/
 class sp_instr_opt_meta : public sp_instr
 {
 public:
 
-  uint m_dest;			// Where we will go
-  uint m_cont_dest;             // Where continue handlers will go
+  uint m_dest;			///< Where we will go
+  uint m_cont_dest;             ///< Where continue handlers will go
 
   sp_instr_opt_meta(uint ip, sp_pcontext *ctx)
     : sp_instr(ip, ctx),
@@ -744,14 +854,14 @@ public:
 
 protected:
 
-  sp_instr *m_optdest;		// Used during optimization
-  sp_instr *m_cont_optdest;     // Used during optimization
+  sp_instr *m_optdest;		///< Used during optimization
+  sp_instr *m_cont_optdest;     ///< Used during optimization
 
 }; // class sp_instr_opt_meta : public sp_instr
 
 class sp_instr_jump : public sp_instr_opt_meta
 {
-  sp_instr_jump(const sp_instr_jump &);	/* Prevent use of these */
+  sp_instr_jump(const sp_instr_jump &);	/**< Prevent use of these */
   void operator=(sp_instr_jump &);
 
 public:
@@ -784,7 +894,7 @@ public:
     m_dest= dest;
   }
 
-  /*
+  /**
     Update the destination; used by the optimizer.
   */
   virtual void set_destination(uint old_dest, uint new_dest)
@@ -798,7 +908,7 @@ public:
 
 class sp_instr_jump_if_not : public sp_instr_jump
 {
-  sp_instr_jump_if_not(const sp_instr_jump_if_not &); /* Prevent use of these */
+  sp_instr_jump_if_not(const sp_instr_jump_if_not &); /**< Prevent use of these */
   void operator=(sp_instr_jump_if_not &);
 
 public:
@@ -824,7 +934,7 @@ public:
 
   virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
-  /* Override sp_instr_jump's shortcut; we stop here */
+  /** Override sp_instr_jump's shortcut; we stop here */
   virtual uint opt_shortcut_jump(sp_head *sp, sp_instr *start)
   {
     return m_ip;
@@ -841,7 +951,7 @@ public:
 
 private:
 
-  Item *m_expr;			// The condition
+  Item *m_expr;			///< The condition
   sp_lex_keeper m_lex_keeper;
 
 }; // class sp_instr_jump_if_not : public sp_instr_jump
@@ -849,7 +959,7 @@ private:
 
 class sp_instr_freturn : public sp_instr
 {
-  sp_instr_freturn(const sp_instr_freturn &);	/* Prevent use of these */
+  sp_instr_freturn(const sp_instr_freturn &);	/**< Prevent use of these */
   void operator=(sp_instr_freturn &);
 
 public:
@@ -886,7 +996,7 @@ protected:
 
 class sp_instr_hpush_jump : public sp_instr_jump
 {
-  sp_instr_hpush_jump(const sp_instr_hpush_jump &); /* Prevent use of these */
+  sp_instr_hpush_jump(const sp_instr_hpush_jump &); /**< Prevent use of these */
   void operator=(sp_instr_hpush_jump &);
 
 public:
@@ -908,7 +1018,7 @@ public:
 
   virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads);
 
-  /* Override sp_instr_jump's shortcut; we stop here. */
+  /** Override sp_instr_jump's shortcut; we stop here. */
   virtual uint opt_shortcut_jump(sp_head *sp, sp_instr *start)
   {
     return m_ip;
@@ -921,7 +1031,7 @@ public:
 
 private:
 
-  int m_type;			// Handler type
+  int m_type;			///< Handler type
   uint m_frame;
   List<struct sp_cond_type> m_cond;
 
@@ -930,7 +1040,7 @@ private:
 
 class sp_instr_hpop : public sp_instr
 {
-  sp_instr_hpop(const sp_instr_hpop &);	/* Prevent use of these */
+  sp_instr_hpop(const sp_instr_hpop &);	/**< Prevent use of these */
   void operator=(sp_instr_hpop &);
 
 public:
@@ -955,7 +1065,7 @@ private:
 
 class sp_instr_hreturn : public sp_instr_jump
 {
-  sp_instr_hreturn(const sp_instr_hreturn &);	/* Prevent use of these */
+  sp_instr_hreturn(const sp_instr_hreturn &);	/**< Prevent use of these */
   void operator=(sp_instr_hreturn &);
 
 public:
@@ -986,10 +1096,10 @@ private:
 }; // class sp_instr_hreturn : public sp_instr_jump
 
 
-/* This is DECLARE CURSOR */
+/** This is DECLARE CURSOR */
 class sp_instr_cpush : public sp_instr
 {
-  sp_instr_cpush(const sp_instr_cpush &); /* Prevent use of these */
+  sp_instr_cpush(const sp_instr_cpush &); /**< Prevent use of these */
   void operator=(sp_instr_cpush &);
 
 public:
@@ -1005,7 +1115,7 @@ public:
 
   virtual void print(String *str);
 
-  /*
+  /**
     This call is used to cleanup the instruction when a sensitive
     cursor is closed. For now stored procedures always use materialized
     cursors and the call is not used.
@@ -1014,14 +1124,14 @@ public:
 private:
 
   sp_lex_keeper m_lex_keeper;
-  uint m_cursor;                /* Frame offset (for debugging) */
+  uint m_cursor;                /**< Frame offset (for debugging) */
 
 }; // class sp_instr_cpush : public sp_instr
 
 
 class sp_instr_cpop : public sp_instr
 {
-  sp_instr_cpop(const sp_instr_cpop &); /* Prevent use of these */
+  sp_instr_cpop(const sp_instr_cpop &); /**< Prevent use of these */
   void operator=(sp_instr_cpop &);
 
 public:
@@ -1046,7 +1156,7 @@ private:
 
 class sp_instr_copen : public sp_instr
 {
-  sp_instr_copen(const sp_instr_copen &); /* Prevent use of these */
+  sp_instr_copen(const sp_instr_copen &); /**< Prevent use of these */
   void operator=(sp_instr_copen &);
 
 public:
@@ -1066,14 +1176,14 @@ public:
 
 private:
 
-  uint m_cursor;		// Stack index
+  uint m_cursor;		///< Stack index
 
 }; // class sp_instr_copen : public sp_instr_stmt
 
 
 class sp_instr_cclose : public sp_instr
 {
-  sp_instr_cclose(const sp_instr_cclose &); /* Prevent use of these */
+  sp_instr_cclose(const sp_instr_cclose &); /**< Prevent use of these */
   void operator=(sp_instr_cclose &);
 
 public:
@@ -1098,7 +1208,7 @@ private:
 
 class sp_instr_cfetch : public sp_instr
 {
-  sp_instr_cfetch(const sp_instr_cfetch &); /* Prevent use of these */
+  sp_instr_cfetch(const sp_instr_cfetch &); /**< Prevent use of these */
   void operator=(sp_instr_cfetch &);
 
 public:
@@ -1131,7 +1241,7 @@ private:
 
 class sp_instr_error : public sp_instr
 {
-  sp_instr_error(const sp_instr_error &); /* Prevent use of these */
+  sp_instr_error(const sp_instr_error &); /**< Prevent use of these */
   void operator=(sp_instr_error &);
 
 public:
@@ -1221,4 +1331,8 @@ sp_prepare_func_item(THD* thd, Item **it_addr);
 bool
 sp_eval_expr(THD *thd, Field *result_field, Item **expr_item_ptr);
 
+/**
+  @} (end of group Stored_Routines)
+*/
+
 #endif /* _SP_HEAD_H_ */
