@@ -149,7 +149,7 @@ proc_analyse_init(THD *thd, ORDER *param, select_result *result,
       case INT_RESULT:
         // Check if fieldtype is ulonglong
         if (item->type() == Item::FIELD_ITEM &&
-            ((Item_field*) item)->field->type() == FIELD_TYPE_LONGLONG &&
+            ((Item_field*) item)->field->type() == MYSQL_TYPE_LONGLONG &&
             ((Field_longlong*) ((Item_field*) item)->field)->unsigned_flag)
           new_field= new field_ulonglong(item, pc);
         else
@@ -185,8 +185,7 @@ err:
 
 bool test_if_number(NUM_INFO *info, const char *str, uint str_len)
 {
-  const char *begin, *end = str + str_len;
-
+  const char *begin, *end= str + str_len;
   DBUG_ENTER("test_if_number");
 
   /*
@@ -200,8 +199,8 @@ bool test_if_number(NUM_INFO *info, const char *str, uint str_len)
   if (*str == '-')
   {
     info->negative = 1;
-    if (++str == end || *str == '0') // converting -0 to a number
-      DBUG_RETURN(0);		     // might lose information
+    if (++str == end || *str == '0')    // converting -0 to a number
+      DBUG_RETURN(0);                   // might lose information
   }
   else
     info->negative = 0;
@@ -219,31 +218,31 @@ bool test_if_number(NUM_INFO *info, const char *str, uint str_len)
     int error;
     info->ullval= (ulonglong) my_strtoll10(begin, &endpos, &error);
     if (info->integers == 1)
-      DBUG_RETURN(0);		     // a single number can't be zerofill
+      DBUG_RETURN(0);                   // single number can't be zerofill
     info->maybe_zerofill = 1;
-    DBUG_RETURN(1);		     // a zerofill number, or an integer
+    DBUG_RETURN(1);                     // a zerofill number, or an integer
   }
   if (*str == '.' || *str == 'e' || *str == 'E')
   {
-    if (info->zerofill)		     // can't be zerofill anymore
+    if (info->zerofill)                 // can't be zerofill anymore
       DBUG_RETURN(0);
-    if ((str + 1) == end)	     // number was something like '123[.eE]'
+    if ((str + 1) == end)               // number was something like '123[.eE]'
     {
       char *endpos= (char*) str;
       int error;
       info->ullval= (ulonglong) my_strtoll10(begin, &endpos, &error);
       DBUG_RETURN(1);
     }
-    if (*str == 'e' || *str == 'E')  // number may be something like '1e+50'
+    if (*str == 'e' || *str == 'E')     // number may be something like '1e+50'
     {
       str++;
       if (*str != '-' && *str != '+')
-        DBUG_RETURN(0);
+	DBUG_RETURN(0);
       for (str++; str != end && my_isdigit(system_charset_info,*str); str++) ;
       if (str == end)
       {
-	info->is_float = 1;	     // we can't use variable decimals here
-        DBUG_RETURN(1);
+	info->is_float = 1;             // we can't use variable decimals here
+	return 1;
       }
       DBUG_RETURN(0);
     }
@@ -499,7 +498,7 @@ void field_decimal::add()
 
   if (room_in_tree)
   {
-    char buf[DECIMAL_MAX_FIELD_SIZE];
+    uchar buf[DECIMAL_MAX_FIELD_SIZE];
     my_decimal2binary(E_DEC_FATAL_ERROR, dec, buf,
                       item->max_length, item->decimals);
     if (!(element = tree_insert(&tree, (void*)buf, 0, tree.custom_arg)))
@@ -669,7 +668,7 @@ void field_ulonglong::add()
 } // field_ulonglong::add
 
 
-int analyse::send_row(List<Item> &field_list __attribute__((unused)))
+int analyse::send_row(List<Item> & /* field_list */)
 {
   field_info **f = f_info;
 
@@ -765,26 +764,26 @@ int analyse::end_of_records()
     {
       switch (((Item_field*) (*f)->item)->field->real_type())
       {
-      case FIELD_TYPE_TIMESTAMP:
+      case MYSQL_TYPE_TIMESTAMP:
 	ans.append(STRING_WITH_LEN("TIMESTAMP"));
 	break;
-      case FIELD_TYPE_DATETIME:
+      case MYSQL_TYPE_DATETIME:
 	ans.append(STRING_WITH_LEN("DATETIME"));
 	break;
-      case FIELD_TYPE_DATE:
-      case FIELD_TYPE_NEWDATE:
+      case MYSQL_TYPE_DATE:
+      case MYSQL_TYPE_NEWDATE:
 	ans.append(STRING_WITH_LEN("DATE"));
 	break;
-      case FIELD_TYPE_SET:
+      case MYSQL_TYPE_SET:
 	ans.append(STRING_WITH_LEN("SET"));
 	break;
-      case FIELD_TYPE_YEAR:
+      case MYSQL_TYPE_YEAR:
 	ans.append(STRING_WITH_LEN("YEAR"));
 	break;
-      case FIELD_TYPE_TIME:
+      case MYSQL_TYPE_TIME:
 	ans.append(STRING_WITH_LEN("TIME"));
 	break;
-      case FIELD_TYPE_DECIMAL:
+      case MYSQL_TYPE_DECIMAL:
 	ans.append(STRING_WITH_LEN("DECIMAL"));
 	// if item is FIELD_ITEM, it _must_be_ Field_num in this case
 	if (((Field_num*) ((Item_field*) (*f)->item)->field)->zerofill)
@@ -1034,7 +1033,7 @@ String *field_decimal::avg(String *s, ha_rows rows)
 {
   if (!(rows - nulls))
   {
-    s->set((double) 0.0, 1,my_thd_charset);
+    s->set_real((double) 0.0, 1,my_thd_charset);
     return s;
   }
   my_decimal num, avg_val, rounded_avg;
@@ -1055,7 +1054,7 @@ String *field_decimal::std(String *s, ha_rows rows)
 {
   if (!(rows - nulls))
   {
-    s->set((double) 0.0, 1,my_thd_charset);
+    s->set_real((double) 0.0, 1,my_thd_charset);
     return s;
   }
   my_decimal num, tmp, sum2, sum2d;
@@ -1068,7 +1067,7 @@ String *field_decimal::std(String *s, ha_rows rows)
   my_decimal_sub(E_DEC_FATAL_ERROR, &sum2, sum_sqr+cur_sum, &tmp);
   my_decimal_div(E_DEC_FATAL_ERROR, &tmp, &sum2, &num, prec_increment);
   my_decimal2double(E_DEC_FATAL_ERROR, &tmp, &std_sqr);
-  s->set(((double) std_sqr <= 0.0 ? 0.0 : sqrt(std_sqr)),
+  s->set_real(((double) std_sqr <= 0.0 ? 0.0 : sqrt(std_sqr)),
          min(item->decimals + prec_increment, NOT_FIXED_DEC), my_thd_charset);
 
   return s;
@@ -1102,14 +1101,14 @@ int collect_real(double *element, element_count count __attribute__((unused)),
   else
     info->found = 1;
   info->str->append('\'');
-  s.set(*element, info->item->decimals, current_thd->charset());
+  s.set_real(*element, info->item->decimals, current_thd->charset());
   info->str->append(s);
   info->str->append('\'');
   return 0;
 } // collect_real
 
 
-int collect_decimal(char *element, element_count count,
+int collect_decimal(uchar *element, element_count count,
                     TREE_INFO *info)
 {
   char buff[DECIMAL_MAX_STR_LENGTH];
